import os
import unittest

testpath = os.path.dirname(__file__)


class _Test(unittest.TestCase):
    """ Test main module """

    def _test_main(self):
        """ Run main function """
        pass
